#import "Controller.h"

@implementation Controller

/* Test methods */

- exampleWithNoArchiving:sender
{
	id		theGarage;
	id		myTruck;
	
	/* First Put myTruck into theGarage. */
	
	theGarage = [ [Garage alloc] init];
	myTruck = [self getTruck];
	[theGarage setAVehicle:myTruck];
	
	/* Display what is in the Garage. */
	
	[self appendText: "The Garage before the truck was stolen....\n\n"];
	[self printGarage:theGarage];
	
	/* Steal the truck */
	
	[myTruck free];
	
	/* Display what is in the Garage. */
	
	[self appendText: "The Garage after the truck was stolen....\n\n"];
	[self printGarage:theGarage];
	
	[theGarage free];
	
	return self;
}

- exampleWithArchiving:sender
{
	id		theGarage;
	id		myTruck;
	
	/* First Put myTruck into theGarage. */
	
	theGarage = [ [Garage alloc] init];
	myTruck = [self getTruck];
	[theGarage setAVehicle:myTruck];
	
	/* Archive the Garage. */
	
	[self saveGarage:theGarage];
	
	/* Remove all references to the Garage and it's vehicle. */
	
	[self appendText: "Removing the Garage and Truck from memory...\n"];
	
	[theGarage free];
	[myTruck free];
	
	/* Now retrieve the Garage and Truck from the archive file. */
	
	[self appendText: "Retrieving the Garage and Truck from a file...\n"];
	
	theGarage = [self retrieveGarage];
	myTruck = [theGarage aVehicle];
	
	/* Display what is in the Garage. */
	
	[self appendText: "The Garage before the truck was stolen....\n\n"];
	[self printGarage:theGarage];
	
	/* Steal the truck */
	
	[myTruck free];  /* CONSOLE: Unknown error code 10020 in NXReportError. */
	
	/* Display what is in the Garage. */
	
	[self appendText: "The Garage after the truck was stolen....\n\n"];
	[self printGarage:theGarage];
	
	[theGarage free];
	
	return self;
}

- getTruck
{
	id	aTruck;
	
	aTruck = [ [Truck alloc] init];
	[aTruck setMake:"Ford"];
	[aTruck setModel:"Ranger"];
	
	return aTruck;
}

- printGarage:theGarage
{
	id		theVehicle;
	
	theVehicle = [theGarage aVehicle];
	[self appendText: "\nContents of the Garage.....\n\n"];
	if (theVehicle)
		sprintf(buf, "\tVehicle => %s %s\n\n", [theVehicle make], 
			[theVehicle model]);
	else
		sprintf(buf, "\tThe Garage is empty!\n\n");
		
	[self appendText:buf];
	
	return self;
}

- saveGarage:theGarage
{
	NXTypedStream	*writeFile;
	char			filename[BUFSIZ];
	
	sprintf(filename, "%s/Garage", NXHomeDirectory());
	sprintf(buf, "\nAttempting to save Garage in file => '%s'\n", filename);

	[self appendText: buf];

	writeFile = NXOpenTypedStreamForFile(filename, NX_WRITEONLY);
	if (writeFile != (NXTypedStream *) NULL)
	{
		/* A stream and File has been created for the list. */
		
		NXWriteRootObject(writeFile, theGarage);
		NXCloseTypedStream(writeFile);
		
		[self appendText: "The Garage has been saved!\n"];
	}
	else
	{
		sprintf(buf, "\n ERROR (%s): %s\n", filename, strerror(errno));
		[self appendText:buf];
	}
	
	return self;
}

- retrieveGarage
{
	NXTypedStream	*readFile;
	char			filename[BUFSIZ];
	id				theGarage;
	
	sprintf(filename, "%s/Garage", NXHomeDirectory());
	sprintf(buf, "\nAttempting to read file => '%s'\n", filename);

	[self appendText: buf];

	readFile = NXOpenTypedStreamForFile(filename, NX_READONLY);
	if (readFile != (NXTypedStream *) NULL)
	{
		/* File has been read into the stream. */
		
		theGarage =  NXReadObject(readFile);
		NXCloseTypedStream(readFile);
		
		[self appendText: "The Garage has been retrieved!\n"];
	}
	else
	{
		theGarage = nil;
		sprintf(buf, "\n ERROR (%s): %s\n", filename, strerror(errno));
		[self appendText:buf];
	}
	
	return theGarage;
}

/* Controller methods */

- appendText:(const char *)aString
{
	int		length = [text textLength];
	
	[text setSel:length :length];
	[text replaceSel:aString];
	[text scrollSelToVisible];
	[text display];
	
	return self;
}

- clear:sender
{
	[ [text selectAll:self] delete:self];
	
    return self;
}

@end

@implementation Controller(ApplicationDelegate)

- appDidInit:sender
{
	[ [text window] makeKeyAndOrderFront:nil];
	[text selectAll:self];
	
	return self;
}

@end

@implementation Controller(WindowDelegate)

- windowWillClose:sender
{
	[NXApp terminate:nil];
	
	return self;
}

@end
