/* COMPILE: cc move16test.c
 * Test that shows move16 failure under NeXTstep 3.0 & 3.1; works OK on 2.1.
 */
#import <ansi/ansi.h>

void m16cpy(unsigned char *dst, const unsigned char *src, int length)
{
	assert(((unsigned long)dst & 15) == 0 &&
	       ((unsigned long)src & 15) == 0 &&
	       (length & 15) == 0);

	asm("moveql #16,d1\n"
	    "\tbra LL1\n"
	    "LL0:\tmove16 %0+,%1+\n"
	    "LL1:\tsubl d1,%2\n"
		"\tbpl LL0" : : "m" (*src), "m" (*dst), "r" (length) : "d1");
}

inline void *align(void *p) { return (void *)(((long)p + 15) & ~15); }

inline void *alloc(size_t s)
{
	/* leave enough room for alignment */
	void *p = malloc(s + 15);
	if (p == NULL) { fprintf(stderr,"out of memory!\n"); abort(); }
	return p;
}

int main()
{
    size_t usize, size;
    int retval = EXIT_FAILURE;

    for (usize = 16; usize < 10000000; usize = usize*1.17 + 1)
    {
	void *usource, *udest;	/* unaligned versions */
	unsigned char *source, *dest;
	int count;

	size = usize & ~15;

	fprintf(stderr,"%lu\r", (unsigned long)size);	
	source = align(usource = alloc(size));
	dest = align(udest = alloc(size));

	/* fill source area */
	for (count = 0; count < size; count++)
	    source[count] = count;
	
	/* fill dest area with something different */
	for (count = 0; count < size; count++)
	    dest[count] = ~count;

	/* do the copy */
	m16cpy(dest, source, size);

	/* compare the data */
	for (count = 0; count < size; count++)
	{
	    unsigned char d = dest[count], s = source[count];
	    if (d != s)
	    {
		printf("destination %2.2x at %p doesn't match "
		    "source %2.2x at %p (count %d) on size %lu\n",
		    (unsigned)d, (void *)&dest[count],
		    (unsigned)s, (void *)&source[count],
		    count, (unsigned long)size);
		goto error;
	    }
	}
	
	free(usource);
	free(udest);
    }
    printf("move16 test passed!\n");
    retval = EXIT_SUCCESS;
error:
    system("hostinfo");
    return retval;
}
